// ------------------------------------------------------------------ LIBRARIES

#include "board.h"
 
#include "log.h"
#include "g2c.h"
#include "string.h"
#include "hydroprobe.h"
#include "drv_i2c_master.h"
#include "conversions.h"
#include "relay.h"
#include "lightmixsens.h"
#include "environment.h"
#include "drv_analog_in.h"
#include "drv_digital_out.h"

// ------------------------------------------------------------------ DEFINITIONS


// BUffer settings
#define PROCESS_COUNTER 10
#define PROCESS_RX_BUFFER_SIZE 500
#define PROCESS_PARSER_BUFFER_SIZE 500

// AT commands
#define ATE               "ATE1"
#define AT                "AT"
#define AT_CEN            "AT+CEN=1"
#define AT_NWCR           "AT+NWCR"
#define AT_NWC            "AT+NWC=1"
#define AT_BRCR           "AT+BRCR"
#define AT_BRC            "AT+BRC=1"
#define AT_DSET           "AT+DSET"
#define AT_PUB            "AT+PUB"

// Sensor/Actuator reference
#define FAN_REF           "fan_actuator"
#define LIGHT_SWITCH_REF  "light_switch"
#define FAN_ON     "1"
#define FAN_OFF    "0"

#define HUMIDITY_REF            "humidity"
#define TEMPERATURE_REF         "temperature"
#define PRESSURE_REF            "pressure"
#define GND_HUMIDITY_REF        "gnd_humidity"
#define AMBIENT_LIGHT_REF       "ambient_light"
#define WATER_TEMP_REF          "water_temp"
#define LIGHT_STATE_REF         "lights_state"
#define FAN_STATE_REF           "fan_state"

// Operator settings
#define NETWORK_USERNAME  "SSID"
#define NETWORK_PASSWORD  "PASS"

// Broker(device) settings
#define DEVICE_KEY        "dev_key"
#define DEVICE_PASSWORD   "dev_pass"

enum {
    AMBIENT_HUMIDITY = 0,
    AMBIENT_TEMPERATURE,
    AMBIENT_PRESSURE,
    GROUND_MOISTURE,
    WATER_TEMPERATURE,
    AMBIENT_LIGHT
};

// ------------------------------------------------------------------ VARIABLES

// Cloud gateway
static g2c_t g2c;
static log_t logger;

static char data_buf[ 20 ] = "12.2";
static char current_parser_buf[ PROCESS_PARSER_BUFFER_SIZE ];
static uint8_t send_data_cnt = 0;
static uint8_t send_variable = 0;

// Environment sensor
static environment_t environment;
static float temperature;
static float pressure;
static float humidity;


// Ground moisture sensor
static hydroprobe_t hydroprobe;
i2c_master_t i2c_master;

static uint8_t humy_val = 0;
static uint16_t dry_val = 3460;
static uint16_t wet_val = 2028;

float ground_humidity_float;

// Light sensor
static lightmixsens_t lightmixsens;
lightmixsens_data_obj lightmixsens_data;
char prox_str[ 20 ];

// Fan actuator
static relay_t relay;

static int switch_state = 0;


//Water temp sensor
analog_in_t analog_in;
float water_temp;

long int duty_on = 0;
long int duty_off = 0;

//------------------------------------------------------- ADDITIONAL FUNCTIONS

float read_lm35 ( analog_in_t * p_analog_in )
{
    // Read analog input
    long int analog_avg = 0;
    float voltage = 0;
    uint16_t * read_value;
    
    for(int i = 0; i <= 100; i++)
    {
        analog_in_read( p_analog_in, &read_value );
        analog_avg += (int) read_value;
        Delay_ms(1);
    }
    analog_avg = (analog_avg/100);
        
    voltage = (analog_avg * 3.3)/4096;
    
    return (float)voltage * 100;
}

static void g2c_process ( void )
{
    int32_t rsp_size;
    uint16_t rsp_cnt = 0;
    
    char uart_rx_buffer[ PROCESS_RX_BUFFER_SIZE ] = { 0 };
    uint8_t check_buf_cnt;
    uint8_t process_cnt = PROCESS_COUNTER;
    
    // Clear parser buffer
    memset( current_parser_buf, 0 , PROCESS_PARSER_BUFFER_SIZE ); 
    
    while( process_cnt != 0 )
    {
        rsp_size = g2c_generic_read( &g2c, &uart_rx_buffer, PROCESS_RX_BUFFER_SIZE );

        if ( rsp_size > 0 )
        {  
            // Validation of the received data
            for ( check_buf_cnt = 0; check_buf_cnt < rsp_size; check_buf_cnt++ )
            {
                if ( uart_rx_buffer[ check_buf_cnt ] == 0 ) 
                {
                    uart_rx_buffer[ check_buf_cnt ] = 13;
                }
            }
            log_printf( &logger, "UART Buffer", uart_rx_buffer );
            log_printf( &logger, "%s", uart_rx_buffer );

            // Storages data in parser buffer
            rsp_cnt += rsp_size;
            if ( rsp_cnt < PROCESS_PARSER_BUFFER_SIZE )
            {
                strncat( current_parser_buf, uart_rx_buffer, rsp_size );
            }
            
            // Clear RX buffer
            memset( uart_rx_buffer, 0, PROCESS_RX_BUFFER_SIZE );
        } 
        else 
        {
            process_cnt--;
            
            // Process delay 
            Delay_100ms( );
        }
    }
}

static void actuator_demo( char *rsp )
{
    uint8_t sw_state;
    
    uint8_t err = G2C_PARSER_NO_ERROR;
    
    log_printf(&logger, ">>Response: %s \r\n", rsp);
    
    // Actuator switch example
    err = g2c_actuator_sw_parser( rsp, FAN_REF, &sw_state );
    if ( err == G2C_PARSER_NO_ERROR )
    {
        log_printf( &logger, ">> ACTUATOR SW STATE: %d\r\n", sw_state );
        switch_state = sw_state;
        if(switch_state)
        {
            relay_set_state(&relay, 1, RELAY_STATE_ON);
        }
        else
        {
            relay_set_state(&relay, 1, RELAY_STATE_OFF);
        }
    }
    
    err = g2c_actuator_sw_parser( rsp, LIGHT_SWITCH_REF, &sw_state );
    if ( err == G2C_PARSER_NO_ERROR )
    {
        log_printf( &logger, ">> ACTUATOR SW STATE: %d\r\n", sw_state );
        switch_state = sw_state;
        if(switch_state)
        {
            relay_set_state(&relay, 1, RELAY_STATE_ON);
        }
        else
        {
            relay_set_state(&relay, 1, RELAY_STATE_OFF);
        }
    }
}
// ------------------------------------------------------ APPLICATION FUNCTIONS

void application_init ( void )
{
    // Config Objects
    log_cfg_t log_cfg;
    g2c_cfg_t cfg;
    analog_in_config_t  analog_in_cfg;
    environment_cfg_t env_cfg;
    lightmixsens_cfg_t light_cfg;
    relay_cfg_t pump_cfg;
    hydroprobe_cfg_t gnd_cfg;
    
    //I2C setting.(Change MIKROBUS port to match your setup)
    
    i2c_master_config_t i2c_master_cfg;
    i2c_master_configure_default(&i2c_master_cfg);

    i2c_master_cfg.scl = MIKROBUS_1_SCL;
    i2c_master_cfg.sda = MIKROBUS_1_SDA;

    i2c_master_open(&i2c_master, &i2c_master_cfg);
    i2c_master_set_speed(&i2c_master, I2C_MASTER_SPEED_STANDARD);
    

    /** 
     * Logger initialization.
     * Default baud rate: 115200
     * Default log level: LOG_LEVEL_DEBUG
     * @note If USB_UART_RX and USB_UART_TX 
     * are defined as HAL_PIN_NC, you will 
     * need to define them manually for log to work. 
     * See @b LOG_MAP_USB_UART macro definition for detailed explanation.
     */
    LOG_MAP_USB_UART( log_cfg );
    //log_cfg.baud = 57600;
    log_init( &logger, &log_cfg );
    log_info( &logger, "---- Application Init ----" );

    //  Click initialization. (Change MIKROBUS port to match your setup)

    g2c_cfg_setup( &cfg );
    G2C_MAP_MIKROBUS( cfg, MIKROBUS_3 );
    g2c_init( &g2c, &cfg );

    // Power module.

    g2c_module_power( &g2c, true, 0 );
    g2c_reset( &g2c );
    
    // Send setting commands
    
    log_init( &logger, &log_cfg );
    log_printf( &logger, ">> ATE \r\n" );
    g2c_send_command( &g2c, ATE );
    Delay_ms( 3000 );
    g2c_process( );

    log_printf( &logger, ">> AT \r\n" );
    g2c_send_command( &g2c, AT );
    Delay_ms( 3000 );
    g2c_process( );

    log_printf( &logger, ">> AT_CEN \r\n" );
    g2c_send_command( &g2c, AT_CEN );
    g2c_process( );

    log_printf( &logger, ">> NETWORK \r\n" );
    g2c_send_operator_cfg( &g2c, NETWORK_USERNAME, NETWORK_PASSWORD );
    g2c_process( );

    log_printf( &logger, ">> AT NWC \r\n" );
    g2c_send_command( &g2c, AT_NWC );
    g2c_process( );

    log_printf( &logger, ">> DEVICE \r\n" );
    g2c_send_broker_cfg( &g2c, DEVICE_KEY, DEVICE_PASSWORD );
    g2c_process( );

    log_printf( &logger, ">> AT BRC \r\n" );
    g2c_send_command( &g2c, AT_BRC ); 
    g2c_process( );
    
    //  Click ambient light initialization.(Change MIKROBUS port to match your setup)

    lightmixsens_cfg_setup( &light_cfg );
    LIGHTMIXSENS_MAP_MIKROBUS( light_cfg, MIKROBUS_2 );
    lightmixsens_init( &lightmixsens, &light_cfg );

    lightmixsens_default_cfg( &lightmixsens );

    lightmixsens_data.lightmixsens_cdata = LIGHTMIXSENS_DUMMY_DATA;
    lightmixsens_data.lightmixsens_rdata = LIGHTMIXSENS_DUMMY_DATA;
    lightmixsens_data.lightmixsens_gdata = LIGHTMIXSENS_DUMMY_DATA;
    lightmixsens_data.lightmixsens_bdata = LIGHTMIXSENS_DUMMY_DATA;
    lightmixsens_data.lightmixsens_pdata = LIGHTMIXSENS_DUMMY_DATA;
    
    //  Click env initialization.(Change MIKROBUS port to match your setup)

    environment_cfg_setup( &env_cfg );
    ENVIRONMENT_MAP_MIKROBUS( env_cfg, MIKROBUS_1 );
    environment_init( &environment , &env_cfg );
    environment_default_cfg( &environment );
    
    //  Click gnd initialization.(Change MIKROBUS port to match your setup)
    
    hydroprobe_cfg_setup( &gnd_cfg );
    HYDROPROBE_MAP_MIKROBUS( gnd_cfg, MIKROBUS_1 );
    hydroprobe_init( &hydroprobe, &gnd_cfg );
    
    // Relay initialization.(Change MIKROBUS port to match your setup)
    relay_cfg_setup( &pump_cfg );
    RELAY_MAP_MIKROBUS( pump_cfg, MIKROBUS_1 );
    relay_init( &relay, &pump_cfg );

    relay_default_cfg ( &relay );

    analog_in_configure_default( &analog_in_cfg );
    analog_in_cfg.input_pin = PA0;

    analog_in_open( &analog_in, &analog_in_cfg );

    Delay_ms( 3000 );
}

void application_task ( void )
{
    g2c_process( );
    actuator_demo( current_parser_buf );
    
    if ( send_data_cnt == 5 )
    {
        //Updates 1 variable per cycle
        switch (send_variable)
        {
            case AMBIENT_HUMIDITY:
                i2c_master_set_slave_address(&environment.i2c, environment.slave_address);
                err_t status_data;
            
                humidity = environment_get_humidity( &environment );
                log_printf( &logger, "      Humidity : %f%%", humidity);
                
                float_to_str(humidity, data_buf);
                
                log_printf( &logger, ">> HUMIDITY_REF \r\n" );  
                g2c_send_data_ref( &g2c, HUMIDITY_REF, data_buf );
                g2c_process( );

                log_printf( &logger, ">> AT PUB \r\n" );
                g2c_send_command( &g2c, AT_PUB );
                g2c_process( );
                send_variable++;
            break;
            case AMBIENT_TEMPERATURE:
                temperature = environment_get_temperature( &environment);
                log_printf( &logger, " Temperature : %.2fC", temperature);
                
                float_to_str(temperature,data_buf);
                
                log_printf( &logger, ">> TEMPERATURE_REF \r\n" );  
                g2c_send_data_ref( &g2c, TEMPERATURE_REF, data_buf );
                g2c_process( );

                log_printf( &logger, ">> AT PUB \r\n" );
                g2c_send_command( &g2c, AT_PUB );
                g2c_process( );
                send_variable++;
            break;
            case AMBIENT_LIGHT:
                pressure = environment_get_pressure( &environment );
                log_printf( &logger, "      Pressure : %.3fmbar", pressure);
                
                float_to_str(pressure,data_buf);
                
                log_printf( &logger, ">> PRESSURE_REF \r\n" );  
                g2c_send_data_ref( &g2c, PRESSURE_REF, data_buf );
                g2c_process( );

                log_printf( &logger, ">> AT PUB \r\n" );
                g2c_send_command( &g2c, AT_PUB );
                g2c_process( );
                send_variable++;
            break;
            case GROUND_MOISTURE:
                i2c_master_set_slave_address( &hydroprobe.i2c, hydroprobe.slave_address );
        
                humy_val = hydroprobe_rel_env_hum( &hydroprobe, dry_val, wet_val );
                log_printf( &logger, "Environment moisture content: %d %% \r\n ", ( uint16_t ) humy_val );
                log_printf( &logger, "------------------------------\r\n" );
            
                ground_humidity_float = (float) humy_val;

                float_to_str(ground_humidity_float,data_buf);
                
                log_printf( &logger, ">> GND_HUMIDITY_REF \r\n" );  
                g2c_send_data_ref( &g2c, GND_HUMIDITY_REF, data_buf );
                g2c_process( );

                log_printf( &logger, ">> AT PUB \r\n" );
                g2c_send_command( &g2c, AT_PUB );
                g2c_process( );
                send_variable++;
            break;
            case WATER_TEMPERATURE:
                water_temp = read_lm35(&analog_in);
                log_printf( &logger, ">> water_temp %f \r\n", water_temp );  
                float_to_str(water_temp,data_buf);
                
                log_printf( &logger, ">> WATER_TEMP_REF \r\n" );  
                g2c_send_data_ref( &g2c, WATER_TEMP_REF, data_buf );
                g2c_process( );

                log_printf( &logger, ">> AT PUB \r\n" );
                g2c_send_command( &g2c, AT_PUB );
                g2c_process( );
                send_variable++;
            break;
            case AMBIENT_LIGHT:
                i2c_master_set_slave_address( &lightmixsens.i2c, lightmixsens.slave_address );
                
                lightmixsens_wait_atime( &lightmixsens );
            
                lightmixsens_read_word( &lightmixsens, LIGHTMIXSENS_REG_CDATA, 
                                        &lightmixsens_data.lightmixsens_cdata );

                log_printf( &logger, "Lux level : %.2d lx\r\n", lightmixsens_data.lightmixsens_cdata );
                
                float_to_str(lightmixsens_data.lightmixsens_cdata,data_buf);
                
                log_printf( &logger, ">> AMBIENT_LIGHT_REF \r\n" );  
                g2c_send_data_ref( &g2c, AMBIENT_LIGHT_REF, data_buf );
                g2c_process( );

                log_printf( &logger, ">> AT PUB \r\n" );
                g2c_send_command( &g2c, AT_PUB );
                g2c_process( );
                send_variable++;
            break;
            default:
                send_variable = 0;
            break;
        }
    
        
        send_data_cnt = 0;
    }
    Delay_ms(50);
    send_data_cnt++;
    
    if(duty_on == 0)
    {
        relay_set_state(&relay, 2, RELAY_STATE_ON);
        duty_on++;
    }
    else
    {
        duty_on++;
    }
    
    if(duty_on >= 180)
    {
        duty_off++;
        relay_set_state(&relay, 2, RELAY_STATE_OFF);
    }
    
    if(duty_off >= 780)
    {
        duty_on = 0;
        duty_off = 0;
    }
}

void main ( void )
{
    application_init( );
    
    duty_on = 0;
    duty_off = 0;

    for ( ; ; )
    {
        application_task( );
    }
}

// ------------------------------------------------------------------------ END
